/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.featureextraction.mpeg7;

import ij.ImagePlus;

import java.util.List;

import at.lux.imageanalysis.EdgeHistogramImplementation;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DoubleArrayDataRow;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.imagemining.featureextraction.AbstractFeatureExtractionOperator;
import com.rapidminer.imagemining.featureextraction.FeatureExtractionException;
import com.rapidminer.imagemining.io.ImageObject;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.tools.Ontology;
import com.rapidminer.tools.att.AttributeSet;

/**
 * Extractor for MPEG-7 Edge Histogram Feature.
 * 
 * @author Alexander Arimond
 */

public class EdgeHistogramExtractor extends
		AbstractFeatureExtractionOperator<EdgeHistogramDescriptor> {

	public EdgeHistogramExtractor(OperatorDescription description) {
		super(description);
	}

	@Override
	public ExampleSet convertToExampleSet(List<EdgeHistogramDescriptor> features) {
		final int dim = 80;
		AttributeSet set = new AttributeSet();
        final Attribute idAttribute = AttributeFactory.createAttribute("image_ID", Ontology.NOMINAL);
        set.addAttribute(idAttribute);
        final Attribute labelAttribute = AttributeFactory.createAttribute("label", Ontology.NOMINAL);
        set.addAttribute(labelAttribute);
        for (int i = 0; i < dim; i++) {
            set.addAttribute(AttributeFactory.createAttribute("bin" + i, Ontology.NUMERICAL));
        }

        // create table
        final MemoryExampleTable table = new MemoryExampleTable(set.getAllAttributes());
        for (EdgeHistogramDescriptor descr : features){
        	
        	final double [] dataRow = new double[2 + dim];
        	dataRow [0] = idAttribute.getMapping().mapString(descr.getID());
        	dataRow [1] = labelAttribute.getMapping().mapString(descr.getLabel());
            int j = 2;
        	for (int i = 0; i < dim; i++) {
            	dataRow [j] = descr.edgeHistogram[i];
            	j++;
            }
        	table.addDataRow(new DoubleArrayDataRow(dataRow));
        }
        final ExampleSet resultSet = table.createExampleSet(set);
        resultSet.getAttributes().setId(idAttribute);
        resultSet.getAttributes().setLabel(labelAttribute);
        return resultSet;
	}

	@Override
	public EdgeHistogramDescriptor extractFeatures(ImageObject imageObject)
			throws FeatureExtractionException {
		ImagePlus image = new ImagePlus(imageObject.getFilename());
		EdgeHistogramImplementation edge = new EdgeHistogramImplementation(image.getBufferedImage());
		
		EdgeHistogramDescriptor descr = new EdgeHistogramDescriptor(edge.setEdgeHistogram());
		
		descr.setID(imageObject.getID());
		descr.setLabel(imageObject.getLabel());
		
		return descr;
	}

	@Override
	protected MetaData getResultMetaData(ExampleSetMetaData unmodifiedMD) {
		// TODO Auto-generated method stub
		return unmodifiedMD;
	}
	

}
