/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.featureextraction.surf;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Serializable;

import com.rapidminer.imagemining.util.CommandFailedException;
import com.rapidminer.imagemining.util.Util;


/**
 * this uses the SURF library:
 * 
 * Copyright (2006): ETH Zurich, Switzerland
 * Katholieke Universiteit Leuven, Belgium
 * All rights reserved.
 * 
 * For details, see the paper:
 * Herbert Bay,  Tinne Tuytelaars,  Luc Van Gool,
 * "SURF: Speeded Up Robust Features"
 * Proceedings of the ninth European Conference on Computer Vision, May 2006
 * 
 * @author Alexander Arimond
 *
 */



public class SURFFeatureExtractor implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = -7601787714147751475L;
	

	private static File SURF_EXECUTABLE = new File("/home2/a_arimon/invire/InViRe/external/SURF-V1.0.9/surf.ln");
	private static double THRESHOLD = new Double(500);

	public static SURFFeatures extractFeatures(File grayImage, File workingDirectory) throws CommandFailedException, IOException, InterruptedException {
		// FIXME this features.txt is not a thread safe name
		System.out.println("extract features");
		final File featuresFile = new File(workingDirectory, grayImage.getName() + ".features.txt");
		Util.runCommand(SURF_EXECUTABLE.getAbsolutePath() +
				" -i " + grayImage.getAbsolutePath() + 
				" -o " + featuresFile.getAbsolutePath() +
				" -thresh " + THRESHOLD + 
				" -e",  // this takes 128 descriptor values 
				workingDirectory);
		final BufferedReader featuresFileReader = new BufferedReader(new InputStreamReader(new FileInputStream(featuresFile)));
		featuresFileReader.readLine();
		featuresFileReader.readLine();
		SURFFeatures features = new SURFFeatures();
		while (featuresFileReader.ready()){
			String [] values = featuresFileReader.readLine().split(" ");
			double x = Double.parseDouble(values[0]);
			double y = Double.parseDouble(values[1]);
			double [] doubleValues = new double[128];
			assert(values.length == 6 + 128);
			for (int i=6; i<values.length;i++){
				doubleValues[i-6] = Double.parseDouble(values[i]);
			}
			features.addFeature(x, y, doubleValues);
		}
		featuresFileReader.close();
		featuresFile.delete();
		return features;
	}
	
}
