/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.io;

import java.util.LinkedList;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;

/**
 * This operator concatenates two image collections.
 * 
 * @author Alexander Arimond
 */

public class Concatenate extends Operator {

	InputPort in1 = getInputPorts().createPort("images1");
	InputPort in2 = getInputPorts().createPort("images2");
	
	OutputPort out = getOutputPorts().createPort("images");
	
	public Concatenate(OperatorDescription description) {
		super(description);
		this.getTransformer().addGenerationRule(out, ImageObjectList.class);
	}
	
	@Override
	public void doWork() throws OperatorException{
		ImageObjectList images1 = in1.getData();
		ImageObjectList images2 = in2.getData();
		
		LinkedList <ImageObject> images = new LinkedList<ImageObject>();
		images.addAll(images1.getList());
		images.addAll(images2.getList());
		out.deliver(new ImageObjectList(images));
	}

}
