/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.io;

import java.io.File;

import com.rapidminer.imagemining.transformations.AbstractImageTransformationOperator;
import com.rapidminer.imagemining.util.Util;
import com.rapidminer.operator.AbstractIOObject;
import com.rapidminer.operator.Annotations;

/**
 * Representation of a single image. This representation actually references the image on filesystem, 
 * not loads the image into RapidMiner directly.
 * 
 * @author Alexander Arimond
 */

public class ImageObject extends AbstractIOObject {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3077668079732325596L;
	private String filename;
	private String id;
	private String label;
	private String fileType;
	private File thumb;
	

	public ImageObject(String filename, String ID, String label){
		this.filename = filename;
		this.id = ID;
		this.label = label;
		// Thumbnail creation.
		createThumbnail();
		getFileType();
	}
	
	private void createThumbnail() {
		this.thumb = new File(AbstractImageTransformationOperator.workingDir, new File(this.filename).getName() + ".thumb");
		Util.createThumbnail(new File(this.filename), this.thumb);
	}

	public String getFileType(){
		if (this.fileType != null){
			return this.fileType;
			// FIXME check if we have a valid file type here
		} else recognizeFileType();
		return this.fileType;
	}
	
	private void recognizeFileType(){
		fileType = Util.getImageType(new File(filename));
	}
	
	
	
	public String getFilename(){
		return this.filename;
	}
	
	public String getID(){
		return this.id;
	}

	public String getLabel(){
		return this.label;
	}

	public File getThumbnail(){
		return this.thumb;
	}
	
	@Override
	public Annotations getAnnotations() {
		// TODO Auto-generated method stub
		return new Annotations();
	}

}
