/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.io;

import java.io.File;
import java.io.FilenameFilter;
import java.util.LinkedList;
import java.util.List;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeDirectory;
import com.rapidminer.parameter.ParameterTypeString;

public class ReadImages extends Operator {

	private static final String PARAMETER_DIR = "directory";

	private static final String OUTPUTPORT = "images";

	private static final String PARAMETER_LABEL = "label";
	
	private OutputPort outputPort = getOutputPorts().createPort(OUTPUTPORT);
	
	public ReadImages(OperatorDescription description) {
		super(description);
		getTransformer().addGenerationRule(outputPort, ImageObjectList.class);
	}
	
	public void doWork() throws OperatorException{
		File folder = getDirectory();

		File [] files = folder.listFiles(new FilenameFilter() {
			@Override
			public boolean accept(File file, String name) {
				// FIXME hack.. use more sophisticated check method
				if (name.endsWith("jpg") || name.endsWith("png")){
					return true;
				} else {
					return false;
				}
			}
		});
		ImageObjectList images = new ImageObjectList(new LinkedList<ImageObject> ());
		for (int i=0; i < files.length; i++){
			String filename = files [i].getAbsolutePath();
			String label = getLabel();
			String suffix = filename.substring(filename.lastIndexOf("."));
			images.getList().add(new ImageObject(filename, filename.replace(suffix, ""), label));
		}
		outputPort.deliver(images);
		
		
	}
	
	private String getLabel() throws OperatorException{
		String label = getParameterAsString(PARAMETER_LABEL);
		return label;
	}
	
	private File getDirectory() throws OperatorException{
		File file = getParameterAsFile(PARAMETER_DIR);
		if (file.isDirectory()){
			return file;
		} else {
			throw new OperatorException("You must specify a directory as image location!");
		}
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List <ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeDirectory(PARAMETER_DIR, "Set a directory from which to read a collection of images", false));
		types.add(new ParameterTypeString(PARAMETER_LABEL, "Set a label for these images"));
		return types;
	}

}
