/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.transformations;

import java.io.File;
import java.util.List;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeInt;
import com.rapidminer.parameter.UndefinedParameterError;

/**
 * Transforms image into a binary image. Threshold for binarization can be specified.
 * Result format is PGM.
 * 
 * @author Alexander Arimond
 */

public class BinaryOperator extends AbstractImageTransformationOperator {

	private static final String PARAMETER_THRESHOLD = "Threshold";


	public BinaryOperator(OperatorDescription description) {
		super(description);
	}

	@Override
	public String getResultType() {
		//FIXME allow depending on Parameter fileType
		return "pgm";
	}

	@Override
	public String refineID(String ID) {
		return ID + "_bin";
	}

	@Override
	public void transform(File image, File resultFile) {
		int threshold = -1;
		try {
			threshold = getParameterAsInt(PARAMETER_THRESHOLD);
		} catch (UndefinedParameterError e) {
			// cannot occur
		}
		ImageTransformations.transformToBinary(image, resultFile, getResultType(), threshold);
	}
	
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List <ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeInt(PARAMETER_THRESHOLD, "Threshold of for binarization. Values from 0 to 255. Keep field empty for auto-thresholding.", 0, 255, true));
		return types;
	}

}
