/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.transformations;

import java.io.File;
import java.util.List;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.UndefinedParameterError;

/**
 * Flips image either vertically or horizontally. Result type is result of input.
 * 
 * @author Alexander Arimond
 */

public class FlipOperator extends AbstractImageTransformationOperator {
	
	private static final String PARAMETER_DIRECTION = "direction";

	public FlipOperator(OperatorDescription description) {
		super(description);
	}

	@Override
	public String getResultType() {
		// FIXME
		return "jpg";
	}

	@Override
	public String refineID(String ID) {
		if (getDirection().equals(ImageTransformations.DIRECTION_HORIZONTAL)){
			return ID + "_flipH";
		} else {
			return ID + "_flipV";
		}
	}
	
	public String getDirection(){
		try {
			return getParameter(PARAMETER_DIRECTION);
		} catch (UndefinedParameterError e) {
			// cannot occur
			return ImageTransformations.DIRECTION_HORIZONTAL;
		}
	}
	
	@Override
	public void transform(File image, File resultFile) {
		ImageTransformations.flip(image, resultFile, getResultType(), getDirection());
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List <ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeCategory(PARAMETER_DIRECTION, "Flip the images either horizontally or vertically.", new String [] {ImageTransformations.DIRECTION_HORIZONTAL, ImageTransformations.DIRECTION_VERTICAL}, 0));
		return types;
	}
	
}
