/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.transformations;

import java.io.File;
import java.util.List;

import com.rapidminer.imagemining.io.ImageTypes;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.UndefinedParameterError;

/**
 * Transforms images into grayscale images. Bit depth can be specified. Result format is PGM.
 * 
 * @author Alexander Arimond
 */

public class GrayScaleOperator extends AbstractImageTransformationOperator {

	private static String PARAMETER_BIT_DEPTH = "bit depth";
	
	public GrayScaleOperator(OperatorDescription description) {
		super(description);
	}
	
	private int getBitDepth(){
		String bitDepth = "8 Bit";
		try {
			bitDepth = getParameter(PARAMETER_BIT_DEPTH);
		} catch (UndefinedParameterError e) {
			// cannot occur
		}
		if (bitDepth.equals("8 Bit")){
			return ImageTransformations.GRAY_8_BIT;
		}
		if (bitDepth.equals("16 Bit")){
			return ImageTransformations.GRAY_16_BIT;
		}
		if (bitDepth.equals("32 Bit")){
			return ImageTransformations.GRAY_32_BIT;
		} else return ImageTransformations.GRAY_8_BIT; // cannot occur
	}
	
	@Override
	public String refineID(String ID) {
		int bitDepth = getBitDepth();
		if (bitDepth == ImageTransformations.GRAY_8_BIT){
			return ID + "_gray" + 8;
		}
		if (bitDepth == ImageTransformations.GRAY_16_BIT){
			return ID + "_gray" + 16;
		}
		if (bitDepth == ImageTransformations.GRAY_32_BIT){
			return ID + "_gray" + 32;
		}
		return ID + "_gray" + 8; // cannot occur
	}

	@Override
	public void transform(File image, File resultFile) {
		ImageTransformations.transformToGrayScale(image, resultFile, getResultType(), getBitDepth());
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List <ParameterType> types = super.getParameterTypes();
		ParameterTypeCategory type = new ParameterTypeCategory(PARAMETER_BIT_DEPTH, "bit depth", new String[]{"8 Bit", "16 Bit", "32 Bit"}, 0);
		types.add(type);
		return types;
	}

	@Override
	public String getResultType() {
		return ImageTypes.PGM;
	}
	
}
