/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.transformations;

import java.io.File;
import java.util.List;

import com.rapidminer.imagemining.io.ImageTypes;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeDouble;
import com.rapidminer.parameter.UndefinedParameterError;

/**
 * Rotates images clockwise for a specified angle value. Angle can be between 0.0 and 360.0 degree.
 * Result format is same as source format.
 * 
 * @author Alexander Arimond
 */

public class RotateOperator extends AbstractImageTransformationOperator {
	
	private static final String PARAMETER_ANGLE = "Angle";

	public RotateOperator(OperatorDescription description) {
		super(description);
	}

	@Override
	public String getResultType() {
		// FIXME
		return ImageTypes.JPG;
	}

	@Override
	public String refineID(String ID) {
		// TODO Auto-generated method stub
		return ID + "_rotate" + getAngle();
	}

	private double getAngle(){
		double angle = 0.0;
		try {
			angle = getParameterAsDouble(PARAMETER_ANGLE);
			if (angle == Double.NaN)
				angle = 0.0;
		} catch (UndefinedParameterError e) {
			// cannot occur
		} 
//		catch (NullPointerException e){
//			// occurs of some bug: if no value is given in RapidMiner gui
//			// do nothing
//		}
		return angle;
	}
	
	@Override
	public void transform(File image, File resultFile) {
		ImageTransformations.rotate(image, resultFile, getResultType(), getAngle());
	}
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List <ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeDouble(PARAMETER_ANGLE, "Rotation angle (clockwise).", 0.0, 360.0, false));
		return types;
	}

}
