/*
 *  RapidMiner Image Mining Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package com.rapidminer.imagemining.util;

import ij.ImagePlus;
import ij.io.FileSaver;

import java.io.Closeable;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import com.rapidminer.operator.OperatorException;

/**
 * Utilities for Image Mining extension.
 * 
 * @author Alexander Arimond
 */

public class Util {
	
	private static void close(Closeable c) {
	    if (c != null) {
	      try {
	        c.close();
	      } catch (IOException e) {
	        // ignored
	      }
	    }
	  }

	
	public static void runCommand(String command, File directory) throws CommandFailedException, IOException, InterruptedException{
		Process p = null;
		int returnValue = 0;
		try{
			ProcessBuilder pb = new ProcessBuilder(command.split(" "));
			pb.directory(directory);
			p = pb.start();
			returnValue = p.waitFor();
		} finally {
			if (p!=null){
				//System.out.println("close process.");
				close(p.getErrorStream());
				close(p.getInputStream());
				close(p.getOutputStream());
				p.destroy();
				if (returnValue != 0){
					throw new CommandFailedException("Command failed (exit value " + returnValue + ")."); 
				}
			}
		}
		
	}
	
	public static void runCommandWriteToFile(String [] command, File dest, File directory) throws OperatorException, IOException, InterruptedException{
		ProcessBuilder pb = new ProcessBuilder(command);
		pb.directory(directory);
		final Process p = pb.start();
		//BufferedReader std = new BufferedReader(new InputStreamReader(p.getInputStream()));
		//BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(dest)));
		InputStream in = p.getInputStream();
		FileOutputStream out = new FileOutputStream(dest);
		//p.getOutputStream().close();
		//char [] cbuf = new char[512];
 		byte [] bbuf = new byte[512];
		while (true){
			int bytes = in.read(bbuf);
			if (bytes <= 0) break;
			out.write(bbuf);
			/*
			p.getInputStream().read(b)
			out.write
			String line = std.readLine();
 			out.write(line);
 			*/
 			//out.write(cbuf);

 		}
 		int returnValue = p.waitFor();
 		//std.close();
 		in.close();
 		out.close();
 		//int returnValue = p.exitValue();
 		if (returnValue != 0){
 			throw new OperatorException ("command failed (exit value: " + returnValue +"): " + cmdToStr(command));
 		}
	}
	
	
	private static String cmdToStr(String[] command) {
		if (command.length == 0) return "";
		String cmd = command[0];
		for (int i = 1; i < command.length; i++){
			cmd = cmd + " " + command[i];
		}
		return cmd;
	}

	
	// TODO this must be more sophisticated!
	public static String getImageType(File image){
		if (image.getName().endsWith("jpg"))
			return "jpg";
		if (image.getName().endsWith("png"))
			return "png";
		return "jpg";
	}


	public static void createThumbnail(File imageFile, File thumb) {
		if (thumb.isFile()) return;  // in case of thumbnail already exists.
		ImagePlus image = new ImagePlus(imageFile.getAbsolutePath());
		image.getProcessor().resize(80);
		FileSaver saver = new FileSaver(image);
		saver.saveAsJpeg(thumb.getAbsolutePath());
		
		// close image for garbage collection
		image.changes = false;
		image.close();
	}
}
	